package ctbrec.ui.settings;

import java.io.IOException;
import java.lang.reflect.Field;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import ctbrec.Config;
import ctbrec.Settings;
import ctbrec.ui.settings.api.Preferences;
import ctbrec.ui.settings.api.PreferencesStorage;
import javafx.scene.Node;
import javafx.scene.control.CheckBox;
import javafx.scene.control.Label;
import javafx.scene.control.TextField;

public class CtbrecPreferencesStorage implements PreferencesStorage {

    private static final Logger LOG = LoggerFactory.getLogger(CtbrecPreferencesStorage.class);
    public static final String PATTERN_NOT_A_DIGIT = "[^\\d]";
    public static final String COULDNT_SAVE_MSG = "Couldn't save config setting";

    private Config config;
    private Settings settings;

    public CtbrecPreferencesStorage(Config config) {
        this.config = config;
        this.settings = config.getSettings();
    }

    @Override
    public void save(Preferences preferences) throws IOException {
        config.save();
    }

    @Override
    public void load(Preferences preferences) {
        throw new RuntimeException("not implemented");
    }

    @Override
    public Node createGui(String key) throws Exception {
        Field field = Settings.class.getField(key);
        Class<?> t = field.getType();
        Object value = field.get(settings);
        if (t == String.class) {
            return createStringProperty(key, (String) value);
        } else if (t == int.class || t == Integer.class) {
            return createIntegerProperty(key, (Integer) value);
        } else if (t == boolean.class || t == Boolean.class) {
            return createBooleanProperty(key, (Boolean) value);
        } else {
            return new Label("Unsupported Type for key " + key + ": " + t);
        }
    }

    private Node createStringProperty(String fieldName, String value) {
        TextField ctrl = new TextField(value);
        ctrl.textProperty().addListener((obs, oldV, newV) -> saveValue(() -> {
            Field field = Settings.class.getField(fieldName);
            field.set(settings, newV);
            config.save();
        }));
        return ctrl;
    }

    private Node createIntegerProperty(String fieldName, Integer value) {
        TextField ctrl = new TextField(value.toString());
        ctrl.textProperty().addListener((obs, oldV, newV) -> saveValue(() -> {
            if (!newV.matches("\\d*")) {
                ctrl.setText(newV.replaceAll(PATTERN_NOT_A_DIGIT, ""));
            }
            if (!ctrl.getText().isEmpty()) {
                Field field = Settings.class.getField(fieldName);
                field.set(settings, Integer.parseInt(ctrl.getText()));
                config.save();
            }
        }));
        return ctrl;
    }

    private Node createBooleanProperty(String fieldName, Boolean value) {
        CheckBox ctrl = new CheckBox();
        ctrl.setSelected(value);
        ctrl.selectedProperty().addListener((obs, oldV, newV) -> saveValue(() -> {
            Field field = Settings.class.getField(fieldName);
            field.set(settings, newV);
            config.save();
        }));
        return ctrl;
    }

    private void saveValue(Exec exe) {
        try {
            exe.run();
        } catch (Exception e) {
            LOG.error(COULDNT_SAVE_MSG, e);
        }
    }

    @FunctionalInterface
    private interface Exec {
        public void run() throws Exception;
    }
}
